@@ -51,12 +51,21 @@ const QString RelLinksPlugin::CONF_WHEN_NECESSARY = "when necessary";
 
 /** Rellinks factory */
 typedef KGenericFactory<RelLinksPlugin> RelLinksFactory;
+#include <kdeversion.h>
+#if KDE_IS_VERSION(3,2,90)
+#include <kaboutdata.h>
+static const KAboutData aboutdata("rellinks", I18N_NOOP("Rellinks") , "1.0" );
+K_EXPORT_COMPONENT_FACTORY( librellinksplugin, RelLinksFactory(&aboutdata) );
+#else
 K_EXPORT_COMPONENT_FACTORY( librellinksplugin, RelLinksFactory("rellinks") );
+#endif
 
 /** Constructor of the plugin. */
 RelLinksPlugin::RelLinksPlugin(QObject *parent, const char *name,
                            const QStringList &) : KParts::Plugin( parent, name ) {
 
+    setInstance(RelLinksFactory::instance());
+
     // -------- Configuration -------------
     kconfig = new KSimpleConfig("rellinksrc");
     listitem_conf.append(CONF_ALWAYS);
@@ -75,9 +84,9 @@ RelLinksPlugin::RelLinksPlugin(QObject *parent, const char *name,
     m_conf = new KSelectAction( i18n("Configure"), "configure",
                                 KShortcut(), actionCollection(), "rellinks_config" );
     QStringList listitem;
-    listitem.append(i18n("always"));
-    listitem.append(i18n("when necessary"));
-    listitem.append(i18n("never"));
+    listitem.append(i18n("Always"));
+    listitem.append(i18n("When Necessary"));
+    listitem.append(i18n("Never"));
     m_conf->setItems(listitem);
 
     connect(m_conf->popupMenu(), SIGNAL( activated( int ) ), this,
@@ -85,24 +94,22 @@ RelLinksPlugin::RelLinksPlugin(QObject *parent, const char *name,
 
     m_conf->setCurrentItem(mode_index);
 
-    m_url = QString::null;
-
     // ------------- Navigation links --------------
     kaction_map["home"] =  new KAction( i18n("&Top"), "2uparrow", KShortcut("Ctrl+Alt+T"), this, SLOT(goHome()), actionCollection(), "rellinks_top" );
     kaction_map["home"]->setWhatsThis( i18n("<p>This link references a home page or the top of some hierarchy.</p>") );
 
     kaction_map["up"] =  new KAction( i18n("&Up"), "1uparrow", KShortcut("Ctrl+Alt+U"), this, SLOT(goUp()), actionCollection(), "rellinks_up" );
     kaction_map["up"]->setWhatsThis( i18n("<p>This link references the immediate parent of the current document.</p>") );
-    
+
     kaction_map["begin"] =  new KAction( i18n("&First"), "2leftarrow", KShortcut("Ctrl+Alt+F"), this,  SLOT(goFirst()), actionCollection(), "rellinks_first" );
     kaction_map["begin"]->setWhatsThis( i18n("<p>This link type tells search engines which document is considered by the author to be the starting point of the collection.</p>") );
 
     kaction_map["prev"] =  new KAction( i18n("&Previous"), "1leftarrow", KShortcut("Ctrl+Alt+P"), this,  SLOT(goPrevious()), actionCollection(), "rellinks_previous" );
     kaction_map["prev"]->setWhatsThis( i18n("<p>This link references the previous document in an ordered series of documents.</p>") );
-    
+
     kaction_map["next"] =  new KAction( i18n("&Next"), "1rightarrow", KShortcut("Ctrl+Alt+N"), this,  SLOT(goNext()), actionCollection(), "rellinks_next" );
     kaction_map["next"]->setWhatsThis( i18n("<p>This link references the next document in an ordered series of documents.</p>") );
-    
+
     kaction_map["last"] =  new KAction( i18n("&Last"), "2rightarrow", KShortcut("Ctrl+Alt+L"), this,  SLOT(goLast()), actionCollection(), "rellinks_last" );
     kaction_map["last"]->setWhatsThis( i18n("<p>This link references the end of a sequence of documents.</p>") );
 
@@ -160,9 +167,9 @@ RelLinksPlugin::RelLinksPlugin(QObject *parent, const char *name,
     m_more->insert(kaction_map["help"]);
     kaction_map["help"]->setWhatsThis( i18n("<p>This link references the help.</p>") );
 
-    kaction_map["made"]  = new KAction( i18n("&Authors"), "mail_new", KShortcut("Ctrl+Alt+A"), this, SLOT(goAuthor()), actionCollection(), "rellinks_authors" );
-    m_more->insert(kaction_map["made"]);
-    kaction_map["made"]->setWhatsThis( i18n("<p>This link references the author.</p>") );
+    kaction_map["author"]  = new KAction( i18n("&Authors"), "mail_new", KShortcut("Ctrl+Alt+A"), this, SLOT(goAuthor()), actionCollection(), "rellinks_authors" );
+    m_more->insert(kaction_map["author"]);
+    kaction_map["author"]->setWhatsThis( i18n("<p>This link references the author.</p>") );
 
     kaction_map["copyright"]   = new KAction( i18n("Copy&right"), "signature", KShortcut("Ctrl+Alt+R"), this, SLOT(goCopyright()), actionCollection(), "rellinks_copyright" );
     m_more->insert(kaction_map["copyright"]);
@@ -186,7 +193,7 @@ RelLinksPlugin::RelLinksPlugin(QObject *parent, const char *name,
     connect( kactionmenu_map["unclassified"]->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( goAllElements( int ) ) );
     kactionmenu_map["unclassified"]->setDelayed(false);
 
-    // We unativate all the possible actions
+    // We unactivate all the possible actions
     disableAll();
 
     // When the rendering of the HTML is done, we update the site navigation bar
@@ -202,7 +209,7 @@ RelLinksPlugin::RelLinksPlugin(QObject *parent, const char *name,
     connect( m_pollTimer, SIGNAL( timeout() ), this, SLOT( updateToolbar() ) );
 }
 
-/* Destructor */
+/** Destructor */
 RelLinksPlugin::~RelLinksPlugin() {
     delete kconfig;
 }
@@ -241,15 +248,6 @@ void RelLinksPlugin::updateToolbar() {
 
     kdDebug() << "updateToolbar: poller running?=" << m_pollTimer->isActive() << endl;
 
-    // and needs update (Removed because configuration may have changed)
-    /*
-    if ( m_url == part->htmlDocument().URL().string() ) {
-        return;
-    }
-    // Url of the document to test if we need update next time
-    m_url = part->htmlDocument().URL().string();
-    */
-
     // We disable all
     disableAll();
 
@@ -265,97 +263,65 @@ void RelLinksPlugin::updateToolbar() {
         DOM::Element e( linkNodes.item( i ) );
 
 
+        // --- Retrieve of the relation type --
+
         QString rel = e.getAttribute( "rel" ).string();
         rel = rel.simplifyWhiteSpace();
         if (rel.isEmpty()) {
-            // If the "rel" attribut is null then use the "rev" attribute
-            // TODO : Make something to distinct this ?
-            rel =  e.getAttribute( "rev" ).string();
-            rel = rel.simplifyWhiteSpace();
+            // If the "rel" attribut is null then use the "rev" attribute...
+            QString rev = e.getAttribute( "rev" ).string();
+            rev = rev.simplifyWhiteSpace();
+            if (rev.isEmpty()) {
+                // if "rev" attribut is also empty => ignore
+                continue;
+            }
+            // Determine the "rel" equivalent of "rev" type
+            rel =  transformRevToRel(rev);
         }
+        // Determin the name used internally
+        QString lrel = getLinkType(rel.lower());
+        // relation to ignore
+        if (lrel == NULL) continue;
 
-
-        if ( rel.lower() == "stylesheet" )
-            continue; // skip CSS style sheets
-        QString title = e.getAttribute( "title" ).string();
-        QString hreflang = e.getAttribute( "hreflang" ).string();
+        // -- Retrieve of other usefull informations --
 
         QString href = e.getAttribute( "href" ).string();
-        if (href.isEmpty()) continue; // if nowhere to go, ignore the link
+        // if nowhere to go, ignore the link
+        if (href.isEmpty()) continue;
+        QString title = e.getAttribute( "title" ).string();
+        QString hreflang = e.getAttribute( "hreflang" ).string();
 
         KURL ref( part->url(), href );
         if ( title.isEmpty() )
             title = ref.prettyURL();
 
-        QString lrel = rel.lower();
-
-        // Other names used for the same elements
-        lrel = getLinkType(lrel);
+        // -- Menus activation --
 
         // Activation of "Document" menu ?
         if (lrel == "contents" || lrel == "glossary" || lrel == "index" || lrel == "appendix") {
             m_document->setEnabled(true);
         }
-
         // Activation of "More" menu ?
-        if (lrel == "help" || lrel == "made" || lrel == "copyright" ) {
+        if (lrel == "help" || lrel == "author" || lrel == "copyright" ) {
             m_more->setEnabled(true);
         }
 
-        if (lrel == "bookmark") {
+        // -- Buttons or menu items activation / creation --
+        if (lrel == "bookmark" || lrel == "alternate") {
             int id = kactionmenu_map[lrel]->popupMenu()->insertItem( title );
             m_more->setEnabled(true);
             kactionmenu_map[lrel]->setEnabled(true);
-            menu_url_map[lrel][id] = ref;
+            element_map[lrel][id] = e;
 
-        } else if (lrel == "alternate") {
-            int id = kactionmenu_map[lrel]->popupMenu()->insertItem( title );
-            m_more->setEnabled(true);
-            kactionmenu_map[lrel]->setEnabled(true);
-            menu_url_map[lrel][id] = ref;
-
-        } else if (lrel == "appendix") {
-            int id = kactionmenu_map[lrel]->popupMenu()->insertItem( title );
-            kactionmenu_map[lrel]->setEnabled(true);
-            m_document->setEnabled(true);
-            menu_url_map[lrel][id] = ref;
-
-        } else if (lrel == "chapter") {
+        } else if (lrel == "appendix" || lrel == "chapter" || lrel == "section" || lrel == "subsection") {
             int id = kactionmenu_map[lrel]->popupMenu()->insertItem( title );
             m_document->setEnabled(true);
             kactionmenu_map[lrel]->setEnabled(true);
-            menu_url_map[lrel][id] = ref;
-
-        } else if (lrel == "section") {
-            int id = kactionmenu_map[lrel]->popupMenu()->insertItem( title );
-            m_document->setEnabled(true);
-            kactionmenu_map[lrel]->setEnabled(true);
-            menu_url_map[lrel][id] = ref;
-
-        } else if (lrel == "subsection") {
-            int id = kactionmenu_map[lrel]->popupMenu()->insertItem( title );
-            m_document->setEnabled(true);
-            kactionmenu_map[lrel]->setEnabled(true);
-            menu_url_map[lrel][id] = ref;
-
-        } else if (lrel == "stylesheet") {
-            continue; // skip CSS style sheets
-
-        } else if (lrel == "alternate stylesheet") {
-            continue; // skip CSS style sheets
-
-        } else if (lrel == "script") {
-            continue; // skip script
-
-        } else if (lrel == "icon" || lrel == "shortcut icon") {
-            continue; // skip favicon
-
-        } else if (lrel == "prefetch") {
-            continue; // skip prefetch
+            element_map[lrel][id] = e;
 
         } else {
             // It is a unique action
-            url_map[lrel] = ref;
+            element_map[lrel][0] = e;
             if (kaction_map[lrel]) {
                 kaction_map[lrel]->setEnabled(true);
                 // Tooltip
@@ -369,7 +335,7 @@ void RelLinksPlugin::updateToolbar() {
                 // TODO : reference the unknown ?
                 int id = kactionmenu_map["unclassified"]->popupMenu()->insertItem( lrel + " : " + title );
                 kactionmenu_map["unclassified"]->setEnabled(true);
-                menu_url_map["unclassified"][id] = ref;
+                element_map["unclassified"][id] = e;
             }
 
         }
@@ -406,23 +372,30 @@ void RelLinksPlugin::updateToolbar() {
     }
 }
 
-/** Simple links */
-void RelLinksPlugin::goToLink(const QString & link) {
+/** Menu links */
+void RelLinksPlugin::goToLink(const QString & rel, int id) {
     // have the KHTML part open it
     KHTMLPart *part = dynamic_cast<KHTMLPart *>(parent());
     if (!part)
         return;
 
-    KURL url = url_map[link];
+    DOM::Element e = element_map[rel][id];
+    QString href = e.getAttribute("href").string();
+    KURL url( part->url(), href );
+    QString target = e.getAttribute("target").string();
+
+    // URL arguments
+    KParts::URLArgs args;
+    args.frameName = target;
 
     // Add base url if not valid
     if (url.isValid()) {
-        part->browserExtension()->openURLRequest(url);
+        part->browserExtension()->openURLRequest(url, args);
     } else {
         KURL baseURL = part->baseURL();
         QString endURL = url.prettyURL();
         KURL realURL = KURL(baseURL, endURL);
-        part->browserExtension()->openURLRequest(realURL);
+        part->browserExtension()->openURLRequest(realURL, args);
     }
 
 }
@@ -472,7 +445,7 @@ void RelLinksPlugin::goSearch() {
 }
 
 void RelLinksPlugin::goAuthor() {
-    goToLink("made");
+    goToLink("author");
 }
 
 
@@ -480,29 +453,6 @@ void RelLinksPlugin::goCopyright() {
     goToLink("copyright");
 }
 
-
-/** Menu links */
-void RelLinksPlugin::goToLink(const QString & link, int id) {
-    // have the KHTML part open it
-    KHTMLPart *part = dynamic_cast<KHTMLPart *>(parent());
-    if (!part)
-        return;
-
-    KURL url = menu_url_map[link][id];
-
-    // Add base url if not valid
-    if (url.isValid()) {
-        part->browserExtension()->openURLRequest(url);
-    } else {
-        KURL baseURL = part->baseURL();
-        QString endURL = url.prettyURL();
-        KURL realURL = KURL(baseURL, endURL);
-        part->browserExtension()->openURLRequest(realURL);
-    }
-
-}
-
-
 void RelLinksPlugin::goBookmark(int id) {
     goToLink("bookmark", id);
 }
@@ -532,8 +482,7 @@ void RelLinksPlugin::goAllElements(int id) {
 }
 
 void RelLinksPlugin::disableAll() {
-    url_map.clear();
-    menu_url_map.clear();
+    element_map.clear();
 
     // Clear actions
     KActionMap::Iterator it;
@@ -569,12 +518,20 @@ void RelLinksPlugin::changeConf(int id) {
     kconfig->sync();
     kconfig->reparseConfiguration();
 
-    m_url = QString::null;
     this->updateToolbar();
 }
 
-
 QString RelLinksPlugin::getLinkType(const QString &lrel) {
+    // Relations to ignore...
+    if (lrel == "stylesheet"
+          || lrel == "alternate stylesheet"
+          || lrel == "script"
+          || lrel == "icon"
+          || lrel == "shortcut icon"
+          || lrel == "prefetch" )
+        return NULL;
+
+    // ...known relations...
     if (lrel == "top" || lrel == "origin" || lrel == "start")
         return "home";
     if (lrel == "parent")
@@ -593,16 +550,34 @@ QString RelLinksPlugin::getLinkType(const QString &lrel) {
         return "search";
     if (lrel == "alternative stylesheet")
         return "alternate stylesheet";
-    if (lrel == "authors" || lrel == "author" )
-        return "made";
+    if (lrel == "authors")
+        return "author";
     if (lrel == "toc")
         return "contents";
-    if (lrel == "appendice")
-        return "appendix";
 
+    //...unknown relations or name that don't need to change
     return lrel;
 }
 
+QString RelLinksPlugin::transformRevToRel(const QString &rev) {
+    QString altRev = getLinkType(rev);
+
+    // Known relations
+    if (altRev == "prev")
+        return getLinkType("next");
+    if (altRev == "next")
+        return getLinkType("prev");
+    if (altRev == "made")
+        return getLinkType("author");
+    if (altRev == "up")
+        return getLinkType("child");
+    if (altRev == "sibling")
+        return getLinkType("sibling");
+
+    //...unknown inverse relation => ignore for the moment
+    return NULL;
+}
+
 /**
  * Focus event (TODO)
  */
